# from itential.modules.version2021._app_form_builder import AppFormBuilder2021
#
#
# class AppFormBuilder2022(AppFormBuilder2021):
#     def cool_build_form_meth(*args, **kwargs):
#         print('build form meth 2022')
