"""
Link to Itential Docs: https://apidocs.itential.com/2020.2/api/app-nso_manager/

Implemented  Doc String  Tests
    [ ]         [ ]       [ ]   addDeviceForm
    [ ]         [ ]       [ ]   addLockItem
    [ ]         [ ]       [ ]   applyTemplates
    [ ]         [ ]       [ ]   deleteQueueItem
    [ ]         [ ]       [ ]   getAllAuthGroups
    [ ]         [ ]       [ ]   getAllNEDs
    [ ]         [ ]       [ ]   getAuthGroups
    [ ]         [ ]       [ ]   getCommitQueueDeep
    [ ]         [ ]       [ ]   getDevicesFiltered
    [ ]         [ ]       [ ]   getDevicesFilteredForAdapter
    [ ]         [ ]       [ ]   getNEDs
    [ ]         [ ]       [ ]   getQueuedDevices
    [ ]         [ ]       [ ]   getQueueItemDetails
    [ ]         [ ]       [ ]   isAlive
    [ ]         [ ]       [ ]   lockQueueItem
    [ ]         [ ]       [ ]   pruneDevicesAllItems
    [ ]         [ ]       [ ]   pruneDevicesFromItem
    [ ]         [ ]       [ ]   restAction
    [ ]         [ ]       [ ]   restQuery
    [ ]         [ ]       [ ]   runAction
    [ ]         [ ]       [ ]   runCommand
    [ ]         [ ]       [ ]   runCommands
    [ ]         [ ]       [ ]   SetDeviceConfiguration
    [ ]         [ ]       [ ]   setItemNacmGroup
    [ ]         [ ]       [ ]   setLeaf
    [ ]         [ ]       [ ]   unlockQueueItem
    [ ]         [ ]       [ ]   verifyConfig
"""
