"""
Link to Itential Docs: https://apidocs.itential.com/2020.2/api/app-configuration_manager/

Implemented  Doc String  Tests
    [ ]         [ ]       [ ]   adapterProxy
    [ ]         [ ]       [ ]   addDevicesToGroup
    [ ]         [ ]       [ ]   addDevicesToNode
    [ ]         [ ]       [ ]   addTasksToNode
    [ ]         [ ]       [ ]   advancedAutoRemediation
    [ ]         [ ]       [ ]   backUpDevice
    [ ]         [ ]       [ ]   buildSpecLines
    [ ]         [ ]       [ ]   createConfigSpec
    [ ]         [ ]       [ ]   createDeviceGroup
    [ ]         [ ]       [ ]   createGoldenConfigNode
    [ ]         [ ]       [ ]   createGoldenConfigTree
    [ ]         [ ]       [ ]   createGoldenConfigTreeVersion
    [ ]         [ ]       [ ]   createJSONSpec
    [ ]         [ ]       [ ]   createTaskInstance
    [ ]         [ ]       [ ]   deleteDeviceBackups
    [ ]         [ ]       [ ]   deleteDeviceGroups
    [ ]         [ ]       [ ]   deleteDeviceGroupsByName
    [ ]         [ ]       [ ]   deleteGoldenConfigNode
    [ ]         [ ]       [ ]   deleteGoldenConfigTree
    [ ]         [ ]       [ ]   deleteGoldenConfigTrees
    [ ]         [ ]       [ ]   deleteGoldenConfigTreeVersion
    [ ]         [ ]       [ ]   deletePins
    [ ]         [ ]       [ ]   deleteTaskInstances
    [ ]         [ ]       [ ]   deleteVariables
    [ ]         [ ]       [ ]   DiffViewer
    [ ]         [ ]       [ ]   exportGoldenConfigTree
    [ ]         [ ]       [ ]   getAdapterTask
    [ ]         [ ]       [ ]   getAdapterTasks
    [ ]         [ ]       [ ]   getBackups
    [ ]         [ ]       [ ]   getComplianceReportDetail
    [ ]         [ ]       [ ]   getComplianceReportDeviceHistory
    [ ]         [ ]       [ ]   getComplianceReportNodeSummary
    [ ]         [ ]       [ ]   getComplianceReportsByBatch
    [ ]         [ ]       [ ]   getComplianceReportsDetail
    [ ]         [ ]       [ ]   getComplianceReportTaskHistory
    [ ]         [ ]       [ ]   getComplianceReportTreeSummary
    [ ]         [ ]       [ ]   getConfigSpec
    [ ]         [ ]       [ ]   getConfigTemplate
    [ ]         [ ]       [ ]   getDevice
    [ ]         [ ]       [ ]   getDeviceBackupById
    [ ]         [ ]       [ ]   getDeviceConfig
    [ ]         [ ]       [ ]   getDeviceGroupById
    [ ]         [ ]       [ ]   getDeviceGroupByName
    [ ]         [ ]       [ ]   getDeviceGroups
    [ ]         [ ]       [ ]   getDevicesFiltered
    [ ]         [ ]       [ ]   getGoldenConfigTree
    [ ]         [ ]       [ ]   getGoldenConfigTrees
    [ ]         [ ]       [ ]   getGoldenConfigTreeVersion
    [ ]         [ ]       [ ]   getJSONComplianceReportDetail
    [ ]         [ ]       [ ]   getJSONComplianceReportsByBatch
    [ ]         [ ]       [ ]   getJSONSpec
    [ ]         [ ]       [ ]   getJSONSpecWithInheritance
    [ ]         [ ]       [ ]   getPins
    [ ]         [ ]       [ ]   getTaskInstances
    [ ]         [ ]       [ ]   getTopIssues
    [ ]         [ ]       [ ]   getTopIssuesJson
    [ ]         [ ]       [ ]   getTreesForDevice
    [ ]         [ ]       [ ]   gradeComplianceReport
    [ ]         [ ]       [ ]   gradeComplianceReports
    [ ]         [ ]       [ ]   gradeDeviceComplianceHistory
    [ ]         [ ]       [ ]   gradeTaskComplianceHistory
    [ ]         [ ]       [ ]   handlePin
    [ ]         [ ]       [ ]   importBackup
    [ ]         [ ]       [ ]   importGoldenConfigTree
    [ ]         [ ]       [ ]   importGroup
    [ ]         [ ]       [ ]   isAlive
    [ ]         [ ]       [ ]   lookupDiff
    [ ]         [ ]       [ ]   ManualRemediation
    [ ]         [ ]       [ ]   ManualRemediationResults
    [ ]         [ ]       [ ]   patchDeviceConfiguration
    [ ]         [ ]       [ ]   removeDevicesFromGroup
    [ ]         [ ]       [ ]   removeDevicesFromNode
    [ ]         [ ]       [ ]   removeTasksFromNode
    [ ]         [ ]       [ ]   renderJinja2
    [ ]         [ ]       [ ]   runAdapterTask
    [ ]         [ ]       [ ]   runAutoRemediation
    [ ]         [ ]       [ ]   runCompliance
    [ ]         [ ]       [ ]   runComplianceForDevice
    [ ]         [ ]       [ ]   runComplianceForNode
    [ ]         [ ]       [ ]   runComplianceForTree
    [ ]         [ ]       [ ]   runTaskInstance
    [ ]         [ ]       [ ]   searchDeviceGroups
    [ ]         [ ]       [ ]   searchGroups
    [ ]         [ ]       [ ]   translateConfigSpec
    [ ]         [ ]       [ ]   updateConfigSpec
    [ ]         [ ]       [ ]   updateDeviceBackupById
    [ ]         [ ]       [ ]   updateDeviceGroups
    [ ]         [ ]       [ ]   updateGoldenConfigNode
    [ ]         [ ]       [ ]   updateGoldenConfigTree
    [ ]         [ ]       [ ]   updateGoldenConfigTreeVersion
    [ ]         [ ]       [ ]   updateJSONSpec
    [ ]         [ ]       [ ]   updateTaskInstance
"""
