"""
Link to Itential Docs: https://apidocs.itential.com/2020.2/api/app-operations_manager/

Implemented  Doc String  Tests
    [ ]         [ ]       [ ]   cloneAutomation
    [ ]         [ ]       [ ]   createAutomation
    [ ]         [ ]       [ ]   createTrigger
    [ ]         [ ]       [ ]   deleteAutomation
    [ ]         [ ]       [ ]   deleteTrigger
    [ ]         [ ]       [ ]   deleteTriggersByActionId
    [ ]         [ ]       [ ]   exportAutomation
    [ ]         [ ]       [ ]   exportTrigger
    [ ]         [ ]       [ ]   getAutomation
    [ ]         [ ]       [ ]   getAutomations
    [ ]         [ ]       [ ]   getEventDefinition
    [ ]         [ ]       [ ]   getEvents
    [ ]         [ ]       [ ]   getTrigger
    [ ]         [ ]       [ ]   getTriggers
    [ ]         [ ]       [ ]   importAutomations
    [ ]         [ ]       [ ]   importTriggers
    [ ]         [ ]       [ ]   runEndpointTriggerWithPost
    [ ]         [ ]       [ ]   runManualTrigger
    [ ]         [ ]       [ ]   updateAutomation
    [ ]         [ ]       [ ]   updateTrigger

"""
