"""
Link to Itential Docs: https://apidocs.itential.com/2020.2/api/app-template_builder/

Implemented  Doc String  Tests
    [ ]         [ ]       [ ]   applyTemplates
    [ ]         [ ]       [ ]   parseTemplate
    [ ]         [ ]       [ ]   renderJinjaTemplate
"""
