"""
Link to Itential Docs: https://apidocs.itential.com/2020.2/api/app-workflow_engine/

Implemented  Doc String  Tests
    [ ]         [ ]       [ ]   createAccount
    [ ]         [ ]       [ ]   createAdapter
    [ ]         [ ]       [ ]   createGroup
    [ ]         [ ]       [ ]   createIntegration
    [ ]         [ ]       [ ]   createIntegrationModel
    [ ]         [ ]       [ ]   createProfile
    [ ]         [ ]       [ ]   deleteAccount
    [ ]         [ ]       [ ]   deleteAdapter
    [ ]         [ ]       [ ]   deleteGroup
    [ ]         [ ]       [ ]   deleteIntegration
    [ ]         [ ]       [ ]   deleteIntegrationModel
    [ ]         [ ]       [ ]   deleteProfile
    [ ]         [ ]       [ ]   exportPrebuilt
    [ ]         [ ]       [ ]   getAccount
    [ ]         [ ]       [ ]   getAccounts
    [ ]         [ ]       [ ]   getAdapter
    [ ]         [ ]       [ ]   getAdapterChangelogs
    [ ]         [ ]       [ ]   getAdapterHealth
    [ ]         [ ]       [ ]   getAdapterModelTypes
    [ ]         [ ]       [ ]   getAdapters
    [ ]         [ ]       [ ]   getAdapterSchema
    [ ]         [ ]       [ ]   getAdaptersHealth
    [ ]         [ ]       [ ]   getApplication
    [ ]         [ ]       [ ]   getApplicationChangelogs
    [ ]         [ ]       [ ]   getApplicationHealth
    [ ]         [ ]       [ ]   getApplications
    [ ]         [ ]       [ ]   getApplicationSchema
    [ ]         [ ]       [ ]   getApplicationsHealth
    [ ]         [ ]       [ ]   getAssignableRoles
    [ ]         [ ]       [ ]   getGroup
    [ ]         [ ]       [ ]   getGroups
    [ ]         [ ]       [ ]   getIntegration
    [ ]         [ ]       [ ]   getIntegrationModel
    [ ]         [ ]       [ ]   getIntegrationModels
    [ ]         [ ]       [ ]   getIntegrations
    [ ]         [ ]       [ ]   getIntegrationSchema
    [ ]         [ ]       [ ]   getPrebuilt
    [ ]         [ ]       [ ]   getPrebuilts
    [ ]         [ ]       [ ]   getProfile
    [ ]         [ ]       [ ]   getProfiles
    [ ]         [ ]       [ ]   getProfileSchema
    [ ]         [ ]       [ ]   getRepositoryPrebuilts
    [ ]         [ ]       [ ]   getServerHealth
    [ ]         [ ]       [ ]   getSystemHealth
    [ ]         [ ]       [ ]   importPrebuilt
    [ ]         [ ]       [ ]   removePrebuilt
    [ ]         [ ]       [ ]   restartAdapter
    [ ]         [ ]       [ ]   restartApplication
    [ ]         [ ]       [ ]   setAccountGroups
    [ ]         [ ]       [ ]   setAccountPassword
    [ ]         [ ]       [ ]   setGroupRoles
    [ ]         [ ]       [ ]   startAdapter
    [ ]         [ ]       [ ]   startApplication
    [ ]         [ ]       [ ]   stopAdapter
    [ ]         [ ]       [ ]   stopApplication
    [ ]         [ ]       [ ]   switchActiveProfile
    [ ]         [ ]       [ ]   updateAdapter
    [ ]         [ ]       [ ]   updateAdapterLogging
    [ ]         [ ]       [ ]   updateAdapterProperties
    [ ]         [ ]       [ ]   updateApplication
    [ ]         [ ]       [ ]   updateApplicationLogging
    [ ]         [ ]       [ ]   updateApplicationProperties
    [ ]         [ ]       [ ]   updateIntegration
    [ ]         [ ]       [ ]   updateIntegrationModel
    [ ]         [ ]       [ ]   updateIntegrationProperties
    [ ]         [ ]       [ ]   updatePrebuilt
    [ ]         [ ]       [ ]   updateProfile
    [ ]         [ ]       [ ]   validateIntegrationModel
    [ ]         [ ]       [ ]   validatePrebuilt
"""
