"""
Link to Itential Docs: https://apidocs.itential.com/2020.2/api/app-task_worker/

Implemented  Doc String  Tests
    [ ]         [ ]       [ ]   activate
    [ ]         [ ]       [ ]   deactivate
    [ ]         [ ]       [ ]   isActive
"""
