"""
Link to Itential Docs: https://apidocs.itential.com/2020.2/api/app-mop/

Implemented  Doc String  Tests
    [ ]         [ ]       [ ]   chooseDevice
    [ ]         [ ]       [ ]   createAnalyticTemplate
    [ ]         [ ]       [ ]   createTemplate
    [ ]         [ ]       [ ]   deleteAnalyticTemplate
    [ ]         [ ]       [ ]   deleteTemplate
    [ ]         [ ]       [ ]   diffConfig
    [ ]         [ ]       [ ]   exportTemplate
    [ ]         [ ]       [ ]   GetBootFlash
    [ ]         [ ]       [ ]   getDeviceObjectsFiltered
    [ ]         [ ]       [ ]   getDevicesFiltered
    [ ]         [ ]       [ ]   getDevicesFilteredDetailedResults
    [ ]         [ ]       [ ]   getDiff
    [ ]         [ ]       [ ]   importTemplate
    [ ]         [ ]       [ ]   listAnalyticTemplates
    [ ]         [ ]       [ ]   listAnAnalyticTemplate
    [ ]         [ ]       [ ]   listATemplate
    [ ]         [ ]       [ ]   listTemplates
    [ ]         [ ]       [ ]   passThru
    [ ]         [ ]       [ ]   reattempt
    [ ]         [ ]       [ ]   reloadFailed
    [ ]         [ ]       [ ]   reviewSummary
    [ ]         [ ]       [ ]   runAnalyticsTemplate
    [ ]         [ ]       [ ]   runAnalyticsTemplateDevices
    [ ]         [ ]       [ ]   RunCommand
    [ ]         [ ]       [ ]   RunCommandDevices
    [ ]         [ ]       [ ]   RunCommandTemplate
    [ ]         [ ]       [ ]   RunCommandTemplateSingleCommand
    [ ]         [ ]       [ ]   runTemplatesDiff
    [ ]         [ ]       [ ]   updateAnalyticTemplate
    [ ]         [ ]       [ ]   updateTemplate
    [ ]         [ ]       [ ]   verifyConfig
    [ ]         [ ]       [ ]   viewTemplateResults
"""
