# from itential.modules.version2020._app_form_builder import AppFormBuilder2020
#
#
# class AppFormBuilder2021(AppFormBuilder2020):
#     def build_form_meth(*args, **kwargs):
#         print('DEPRECIATED build form meth 2021. '
#               'New method is "cool_build_form_func')
#
#     def cool_build_form_meth(*args, **kwargs):
#         print('cool build form func 2021')
