"""
Link to Itential Docs: https://apidocs.itential.com/2020.2/api/app-service_catalog/

Implemented  Doc String  Tests
    [ ]         [ ]       [ ]   AddNewServiceToCatalog
    [ ]         [ ]       [ ]   deleteService
    [ ]         [ ]       [ ]   GetFormData
    [ ]         [ ]       [ ]   GetFormId
    [ ]         [ ]       [ ]   GetForms
    [ ]         [ ]       [ ]   getGroups
    [ ]         [ ]       [ ]   GetUserObject
    [ ]         [ ]       [ ]   GetWorkflows
    [ ]         [ ]       [ ]   invokeServiceOrder
    [ ]         [ ]       [ ]   ServiceCatalogStore
    [ ]         [ ]       [ ]   ServiceModels
    [ ]         [ ]       [ ]   UpdateServiceInCatalog
"""
