"""
Link to Itential Docs: https://apidocs.itential.com/2020.2/api/app-form_builder/

Implemented  Doc String  Tests
    [ ]         [ ]       [ ]   createFormGroupEntry
    [ ]         [ ]       [ ]   deleteForm
    [ ]         [ ]       [ ]   deleteFormGroups
    [ ]         [ ]       [ ]   exportForm
    [ ]         [ ]       [ ]   fetchData
    [ ]         [ ]       [ ]   getElementDefinition
    [ ]         [ ]       [ ]   getForm
    [ ]         [ ]       [ ]   getFormByName
    [ ]         [ ]       [ ]   importForm
    [ ]         [ ]       [ ]   listElements
    [ ]         [ ]       [ ]   listFormGroups
    [ ]         [ ]       [ ]   listForms
    [ ]         [ ]       [ ]   preserveFormData
    [ ]         [ ]       [ ]   ProviderForm
    [ ]         [ ]       [ ]   removeFormGroup
    [ ]         [ ]       [ ]   replaceFormGroups
    [ ]         [ ]       [ ]   saveForm
    [ ]         [ ]       [ ]   searchForms
    [ ]         [ ]       [ ]   ShowFormByName
    [ ]         [ ]       [ ]   ShowFormPreserveData
"""
